#!/bin/sh
# $1-sysfs path

### run-time variables
CFGDIR=~/.config/pmcputemp
CFG=$CFGDIR/pmcputemprc

### platform-specific sensor finder: stdout
find_sensor_generic() {
	for p in /sys/class/hwmon/*; do
		for q in $p/temp*input; do
			case "$q" in
				*"temp*input") continue ;;
				*) if read pp < $q; then
				       echo $q; return 0
				   fi ;;
			esac
		done
	done

	# 3.2 platforms and pcis
	for q in \
		$(find /sys/devices/platform/ /sys/devices/pci* -type f -name 'temp*_input')
	do
		if read pp < $q; then
			echo $q; return 0
		fi
	done
	return 1
}

# stdout
find_sensor_AuthenticAMD() {
	grep -H . /sys/class/hwmon/*/name | sed '/k10temp/!d; s|/name.*|/temp1_input|'
}

# stdout
find_sensor_GenuineIntel() {
		grep -H . /sys/class/hwmon/*/name | sed '/coretemp/!d; s|/name.*|/temp1_input|'
}

# The Apple system management controller provides _lots_
# of thermal probes, and it's easier to use the `sensors`
# program to watch these probes. Documentation is scarce.
# Function not used, as the AppleSMC is found on Intel-based
# Macs, where GenuineIntel sensors can also be found.
# stdout
find_sensor_AppleSMC() {
		grep -H . /sys/devices/platform/*/name | sed '/applesmc/!d; s|/name.*|/temp1_input|'
}


### generic helpers

# stdout
get_platform() {
	# many ways to get, either dmidecode, device tree, sysfs, dsdt, lscpu, etc
	# we use the simplest one until there is a need for the more complex one
	lscpu | sed '/^Vendor/!d; s/.*: *//'
}

# in: $1-platform name
find_sensor() {
	case "$PLATFORM" in
		AuthenticAMD|GenuineIntel) eval $(echo find_sensor_$PLATFORM) ;;
		generic|*) find_sensor_generic ;; 
	esac
}


### main

### main ###
# $1-sysfs path

# 1. if path specified, we assume user know what he's doing
mkdir -p $CFGDIR
[ "$1" ] && echo "$1" > $CFG && exit 0

# 2. find the platform
printf "Finding platform : "
PLATFORM=$(get_platform)
echo $PLATFORM

# 3. The find the sensor
printf "Finding sensor   : "
if SENSOR=$(find_sensor $PLATFORM); then
	echo $SENSOR

	# 4. save sensor path
	echo $SENSOR > $CFG
	exit 0

else
	echo "not found!"
	exit 1
fi
